@@ -67,6 +67,10 @@ static struct {
 		.cmd	= ETHTOOL_MSG_CABLE_TEST_TDR_NTF,
 		.cb	= cable_test_tdr_ntf_cb,
 	},
+	{
+		.cmd	= ETHTOOL_MSG_FEC_NTF,
+		.cb	= fec_reply_cb,
+	},
 };
 
 static void clear_filter(struct nl_context *nlctx)
@@ -298,6 +302,8 @@ void nl_monitor_usage(void)
 	      stdout);
 	fputs("                ( [ --all ]", stdout);
 	for (i = 1; i < MNL_ARRAY_SIZE(monitor_opts); i++) {
+		if (!strcmp(monitor_opts[i].pattern, monitor_opts[i - 1].pattern))
+			continue;
 		fputs("\n                  | ", stdout);
 		for (p = monitor_opts[i].pattern; *p; p++)
 			if (*p == '|')
